import uuid

import requests
from tqdm import tqdm

from aomame.exceptions import ResponseError

class MicrosoftTranslator:
    """Python SDK for
    https://azure.microsoft.com/en-us/services/cognitive-services/translator/
    """
    def __init__(self, host, key):
        self.host, self.key = host, key
        # See "Add headers" section from
        # https://docs.microsoft.com/en-us/azure/cognitive-services/translator/quickstart-translate?pivots=programming-language-python
        self.headers = {
            'Ocp-Apim-Subscription-Key': key,
            'Content-type': 'application/json',
            'X-ClientTraceId': str(uuid.uuid4())
        }
        # Endpoints.
        self.endpoints = {
            'translate': '/translate?api-version=3.0',
            'transliterate': '/transliterate?api-version=3.0',
            'languages': '/languages?api-version=3.0'
        }
        self.urls = {k:"https://" + self.host + '/' + v
                     for k,v in self.endpoints.items()}

        # Pre-populate the list of languages for translation.
        self._languages = self.languages()

        # Pre-populate the list of languages for transliteration
        self._scripts = self.scripts()

    def api_call(self, operation, method, params=None, json=None):
        """Wrapper class over API calls."""
        url = self.urls[method] + params if params else self.urls[method]
        response = operation(url, headers=self.headers, json=json)
        return response

    def languages(self):
        """Return list of languages available for translation."""
        reponse_json = self.api_call(requests.get, 'languages').json()
        return {lang_code:_dict['name'] for lang_code, _dict in
                reponse_json['translation'].items()}

    def scripts(self):
        """Return list of scripts available for transliteration."""
        _scripts = {}
        reponse_json = self.api_call(requests.get, 'languages').json()
        for l, details in reponse_json['transliteration'].items():
            for s in details['scripts']:
                for _s in s['toScripts']:
                    if _s['name'] == 'Hat':
                        _scripts[_s['code'].lower()] = 'Han Traditional'
                    elif _s['name'] == 'Han':
                        _scripts[_s['code'].lower()] = 'Han Simplified'
                    else:
                        _scripts[_s['code'].lower()] = _s['name']
        return _scripts

    def transliterate(self, text, srclang, from_script, to_script):
        params = f'&language={srclang}&fromScript={from_script}&toScript={to_script}'
        response = self.api_call(requests.post, 'transliterate',
                                 params=params, json=[{'Text': text}])
        if response.status_code == 200:
            return response.json()[0]['text']
        else:
            raise ResponseError(response.json())

    def translate(self, text, srclang, trglang):
        params = f'&to={srclang}&to={trglang}'
        response = self.api_call(requests.post, 'translate',
                                 params=params, json=[{'Text': text}])
        if response.status_code == 200:
            return response.json()[0]['translations'][-1]['text']
        else:
            raise ResponseError(response.json())

    def _get_multiple_translations(self, texts, srclang, trglang):
        params = f'&to={srclang}&to={trglang}'
        # Splitting texts into batches.
        # See https://docs.microsoft.com/en-us/azure/cognitive-services/translator/request-limits
        responses = []
        batch = []
        len_batch = 0
        for t in tqdm(texts):
            if len_batch + len(t) < 5000 and len(batch) < 100:
                batch.append({'Text':t})
                len_batch += len(t)
            else:
                # Process this batch.
                yield requests.post(self.urls['translate'] + params,
                                    headers=self.headers,json=batch)
                # Clear this batch, prepare the next batch.
                batch = [{'Text':t}]
                len_batch = 0
        # Process last batch.
        if batch:
            yield requests.post(self.urls['translate'] + params,
                                headers=self.headers,json=batch)

    def translate_sents(self, texts, srclang, trglang):
        translations = []
        for response in self._get_multiple_translations(texts, srclang, trglang):
            if response.status_code == 200:
                for t in response.json():
                    src = t['translations'][0]['text']
                    trg = t['translations'][1]['text']
                    translations.append((src, trg))
            else:
                raise ResponseError(response.json())
        src, trg = zip(*translations)
        # Sanity check to check that all sentences are translated.
        assert len(trg) == len(texts)
        return trg




from aomame.microsoft import MicrosoftTranslator
from aomame.systran import SystranTranslator
from aomame.google import GoogleTranslator
#from aomame.modernmt import ModernmtTranslator
#from aomame.deepl import DeeplTranslator




import requests
from tqdm import tqdm

from aomame.exceptions import ResponseError

class GoogleTranslator:
    def __init__(self, host, key):
        """Python SDK for
        https://cloud.google.com/translate/docs/apis"""
        # Default host: "translation.googleapis.com"
        self.host, self.key = host, key
        self.headers = {"Content-Type": "application/json; charset=utf-8"}

        self.endpoints = {'translate': f"language/translate/v2?key={self.key}",
                          'languages': f"language/translate/v2/languages?key={self.key}",
                          'detect': f"language/translate/v2/detect?key={self.key}",
                          }

        self.urls = {k:"https://" + self.host + '/' + v for k,v in self.endpoints.items()}

    def api_call(self, operation, method, params=None, json=None):
        """Wrapper class over API calls."""
        # Add other parameters.
        url = self.urls[method] + params if params else self.urls[method]
        response = operation(url, headers=self.headers, json=json)
        return response

    def languages(self):
        response = self.api_call(requests.get, 'languages')
        return set(l['language'] for l in response.json()['data']['languages'])

    def detect(self, text):
        payload = {"q": [text]}
        response = self.api_call(requests.post, 'detect', json=payload)
        return response.json()['data']['detections'][0][0]['language']

    def translate(self, text, srclang, trglang):
        payload = {"q": [text], "target": trglang, "source": srclang, "format": "text"}
        response = self.api_call(requests.post, 'translate', json=payload)
        translation = response.json()['data']['translations'][0]['translatedText']
        return translation

    def _get_multiple_translations(self, texts, srclang, trglang):
        # Splitting texts into batches.
        # See https://cloud.google.com/translate/quotas
        responses = []
        batch = []
        len_batch = 0
        for t in tqdm(texts):
            if len_batch + len(t) < 5000 and len(batch) < 100:
                batch.append(t)
                len_batch += len(t)
            else:
                # Process this batch.
                payload = {"q": batch, "target": trglang, "source": srclang, "format": "text"}
                yield self.api_call(requests.post, 'translate', json=payload)
                # Clear this batch, prepare the next batch.
                batch = [t]
                len_batch = 0
        # Process last batch.
        if batch:
            payload = {"q": batch, "target": trglang, "source": srclang, "format": "text"}
            yield self.api_call(requests.post, 'translate', json=payload)

    def translate_sents(self, texts, srclang, trglang):
        translations = []
        for response in self._get_multiple_translations(texts, srclang, trglang):
            if response.status_code == 200:
                for t in response.json()['data']['translations']:
                    translations.append(t['translatedText'])
            else:
                raise ResponseError(response.json())
        # Sanity check to check that all sentences are translated.
        assert len(texts) == len(translations)
        return translations



from distutils.core import setup
import setuptools


setup(
  name = 'aomame',
  packages = ['aomame'],
  version = '0.0.3',
  description = 'Aomame',
  long_description = 'Pythonic SDK for Translation APIs',
  author = '',
  url = 'https://github.com/alvations/aomame',
  keywords = [],
  classifiers = [
    "Programming Language :: Python :: 3",
    "License :: OSI Approved :: MIT License",
    "Operating System :: OS Independent",
  ],
  install_requires = ['requests', 'tqdm'],
)
